

/*
## Cypress Image Sensor Routines for XXXX sensor (cyu3imagesnesor.c)
## ===========================
##
##  Copyright Cypress Semiconductor Corporation, 2013,
##  All Rights Reserved
##  UNPUBLISHED, LICENSED SOFTWARE.
##
##  CONFIDENTIAL AND PROPRIETARY INFORMATION
##  WHICH IS THE PROPERTY OF CYPRESS.
##
##  Use of this file is governed
##  by the license agreement included in the file
##
##     <install>/license/license.txt
##
##  where <install> is the Cypress software
##  installation root directory path.
##
## ===========================
*/


#include <cyu3error.h>
#include <cyu3i2c.h>
#include "cyu3imagesensor.h"
#include <cyu3utils.h>
#include "cyu3os.h"

/*Uncomment below line to enable UART debug messages during Sensor configuration*/
//#define SENSOR_DEBUG



/* Misc Macros; Fill the correct I2C addresses below */
#define SENSOR_I2C_READ_ADDRESS         (0x35) //Read address = 0xFF 0011 0100
#define SENSOR_I2C_WRITE_ADDRESS        (0x34) //Write address = 0xFE 0011 0101

//#define SENSOR_I2C_READ_ADDRESS         (0x0F) //Read address = 0xFF 0011 0100
//#define SENSOR_I2C_WRITE_ADDRESS        (0x0E) //Write address = 0xFE 0011 0101

/* Register Addresses */


/* Structure to store the Address/Value pair for each
 * register to be updated. */
typedef struct CySensorReg8
{
    uint16_t regAddr;
    uint8_t regValue;
} CySensorReg8_t;

typedef struct CySensorReg16
{
    uint16_t regAddr;
    uint16_t regValue;
} CySensorReg16_t;

typedef struct CySensorReg32
{
    uint16_t regAddr;
    uint32_t regValue;
} CySensorReg32_t;


/* Globals */


/* Configuration Settings*/

/* 1st stabilizaiton period*/
/* Configuration Settings for Sensor Init PAGE63 IN IMX294_DATASHEET*/
CySensorReg8_t SENSOR_timing_power_ConfigurationSettings8[] =
{

	//step1
	{0x3033,0x30},//XMSTA register = 1h, MSTSLV register = 1h)
	{0x303C,0x01},//SYS_MODE register = 1h
//	{0x3000,0x12},//standby_offSTANDBY register = 0h, STBLOGIC register = 1h,STBMIPI register = 0h, STBDV register = 1h


	///////1.3 pll setting///////
	////input_clk=24Mhz///////////
	{0x31E8,0x20},//1LOW
	{0x31E9,0x01},//1HIGH
	{0x3122,0x02},//2
	{0x3129,0x90},//3
	{0x312A,0x02},//4
	{0x311F,0x00},//10
	{0x3123,0x00},//11
	{0x3124,0x00},//12
	{0x3125,0x01},//13
	{0x3127,0x02},//14
	{0x312D,0x02},//15
	//////////////////////////////

	//////1.4 standby cancel///////
	{0x3000,0x12},//standby_offSTANDBY register = 0h, STBLOGIC register = 1h,STBMIPI register = 0h, STBDV register = 1h

	//////1.5 pll release///////
	{0x310B,0x00},//PLLrelease

	/////1.61 initialize communication/////
	//////////////////////////setting PLSTMG//////////////////////////
	{0x3047,0x01},
	{0x304E,0x0B},
	{0x304F,0x24},
	{0x3062,0x25},
	{0x3064,0x78},
	{0x3065,0x33},
	{0x3067,0x71},
	{0x3088,0x75},
	{0x308A,0x09},
	{0x308B,0x01},//low-low hi-hi
	{0x308C,0x61},
	{0x3146,0x00},
	{0x3234,0x32},
	{0x3235,0x00},
	{0x3248,0xBC},
	{0x3249,0x00},
	{0x3250,0xBC},
	{0x3251,0x00},
	{0x3258,0xBC},
	{0x3259,0x00},
	{0x3260,0xBC},
	{0x3261,0x00},///
	{0x3274,0x13},
	{0x3275,0x00},
	{0x3276,0x1F},
	{0x3277,0x00},
	{0x3278,0x30},
	{0x3279,0x00},
	{0x327C,0x13},
	{0x327D,0x00},
	{0x327E,0x1F},
	{0x327F,0x00},
	{0x3280,0x30},
	{0x3281,0x00},
	{0x3284,0x13},
	{0x3285,0x00},
	{0x3286,0x1F},
	{0x3287,0x00},
	{0x3288,0x30},
	{0x3289,0x00},
	{0x328C,0x13},
	{0x328D,0x00},
	{0x328E,0x1F},
	{0x328F,0x00},
	{0x3290,0x30},
	{0x3291,0x00},
	{0x32AE,0x00},
	{0x32AF,0x00},
	{0x32CA,0x5A},
	{0x32CB,0x00},
	{0x332F,0x00},
	{0x334C,0x01},
	{0x335A,0x79},
	{0x335B,0x00},
	{0x335E,0x56},
	{0x335F,0x00},
	{0x3360,0x6A},
	{0x3361,0x00},
	{0x336A,0x56},
	{0x336B,0x00},
	{0x33D6,0x79},
	{0x33D7,0x00},
	{0x340C,0x6E},
	{0x340D,0x00},
	{0x3448,0x7E},
	{0x3349,0x00},

	{0x348E,0x6F},
	{0x348F,0x00},
	{0x3492,0x11},
	{0x34C4,0x5A},
	{0x34C5,0x00},
	{0x3506,0x56},
	{0x3507,0x00},
	{0x350C,0x56},
	{0x350D,0x00},
	{0x350E,0x58},
	{0x350F,0x00},
	{0x3549,0x04},
	{0x355D,0x03},
	{0x355E,0x03},
	{0x3574,0x56},
	{0x3375,0x00},
	{0x3587,0x01},
	{0x35D0,0x5E},
	{0x35D1,0x00},
	{0x35D4,0x63},
	{0x35D5,0x00},
	{0x366A,0x1A},
	{0x366B,0x16},
	{0x366C,0x10},
	{0x366D,0x09},
	{0x366E,0x00},
	{0x366F,0x00},
	{0x3670,0x00},
	{0x3671,0x00},
	{0x3676,0x83},
	{0x3677,0x03},
	{0x3678,0x00},
	{0x3679,0x04},
	{0x367A,0x2C},
	{0x367B,0x05},
	{0x367C,0x00},
	{0x367D,0x06},
	{0x367E,0x00},
	{0x367F,0x07},
	{0x3680,0x4B},
	{0x3681,0x07},
	{0x3690,0x27},
	{0x3691,0x00},
	{0x3692,0x65},
	{0x3693,0x00},
	{0x3694,0x4F},
	{0x3695,0x00},
	{0x3696,0xA1},
	{0x3697,0x00},
	{0x382B,0x68},
	{0x3C00,0x01},
	{0x3C01,0x01},
	{0x3686,0x00},
	{0x3687,0x00},
	{0x36BE,0x01},
	{0x36BF,0x00},
	{0x36C0,0x01},
	{0x36C1,0x00},
	{0x36C2,0x01},
	{0x36C3,0x00},
	{0x36C4,0x01},
	{0x36C5,0x01},
	{0x36C6,0x01},
//////////////////////////done/////////////////////////////////////////////
////////////global timing registers///////////////////////////////////////
	{0x3134,0xAF},
	{0x3135,0x00},//TCLKPOST
	{0x3136,0xC7},
	{0x3137,0x00},//thszero
	{0x3138,0x7F},
	{0x3139,0x00},//THSPREPARE
	{0x313A,0x6F},
	{0x313B,0x00},//TCLKTRAIL
	{0x313C,0x6F},
	{0x313D,0x00},//THSTRAIL
	{0x313E,0xCF},
	{0x313F,0x01},//TCLKZERO
	{0x3140,0x77},
	{0x3141,0x00},//TCLKPREPARE
	{0x3142,0x5F},
	{0x3143,0x00}//TLPX
/////////////////////////DONE//////////////////////////////////

};

/* Configuration Settings for Sensor readout mode PAGE63 IN IMX294_DATASHEET*/
CySensorReg16_t SENSOR_timing_power_ConfigurationSettings16[] =
{

};

/////MODE8 Fr=10.1 HMAX=HMAX min=520=208H  HMAX=600=258  VMAX min=1182 VMAX=6000=1770 SVR=1 SHR=3000=BB8 /////0906H
/////VBLK=VMAX  (SVR value + 1) - minimum VMAX setting  /////PSSLVS4-5H
/////Frame rate [frame/s] = (72 x 106) / {HMAX register value  VMAX register value  (SVR register value + 1) } ///
CySensorReg8_t SENSOR_ReadoutmodeSettings8 [] =
{
	{0x3004,0x4F},//MDSEL1
	{0x3005,0x35},//MDSEL2
	{0x3006,0x00},//MDSEL3
	{0x3007,0xA0},//MDSEL4
	{0x300E,0x01},//SVR             //
	{0x300F,0x00},//SVR 		    //
	{0x3019,0x00},//MDVREV
	{0x302C,0xB8},//SHR			    //
 	{0x302D,0x0B},//SHR			    //
	{0x3030,0x77},//MDSEL5
	{0x3034,0x00},//HOPBOUT_EN
	{0x3035,0x01},//HTRIMMING_EN
	{0x3036,0x30},//HTRIMMING_ START
	{0x3037,0x00},
	{0x3038,0x80},//HTRIMMING_END
	{0x3039,0x10},//HTRIMMING_END_hi
	{0x3068,0x44},//MDSEL15
	{0x3069,0x00},//MDSEL15
	{0x3080,0x00},//MDSEL6
	{0x3081,0x01},//MDSEL7
	{0x3084,0x58},//HCOUNT1=HMAX
	{0x3085,0x02},//HCOUNT1=HMAX
	{0x3086,0x58},//HCOUNT2=HMAX      //
	{0x3087,0x02},//HCOUNT2=HMAX      //
	{0x30A8,0x02},////MDSEL8
	{0x30A9,0x70},//VMAX      //
	{0x30AA,0x17},//VMAX     //
	{0x30AB,0x00},//VMAX     //
	{0x30AC,0x58},//HMAX			   //
	{0x30AD,0x02},//HMAX			   //
	{0x30E2,0x05},////VCUTMODE
	{0x312F,0x04},////OPB_SIZE_V
	{0x3130,0xE4},////WRITE_VSIZE
	{0x3131,0x02},
	{0x3132,0xE0},//Y_OUT_SIZE
	{0x3133,0x02},//Y_OUT_SIZE

	{0x332C,0x42},////PSSLVS1	6000X2-1182=2A42	//
	{0x332D,0x2A},						//
	{0x334A,0x42},////PSSLVS2			//
	{0x334B,0x2A},						//
	{0x357F,0x0C},////MDSEL11
	{0x3580,0x0A},////MDSEL12
	{0x3581,0x0A},////MDSEL13
	{0x3583,0x75},////MDSEL14
	{0x35B6,0x42},////PSSLVS3			//
	{0x35B7,0x2A},						//
	{0x35B8,0x3D},////PSSLVS4-5H	2A3D		//
	{0x35B9,0x2A},						//
	{0x3600,0x90},////MDSEL16
	{0x3601,0x00},
	{0x36BC,0x42},////PSSLVS0			//
	{0x36BD,0x2A},////PSSLVS0			//
	{0x3846,0x00},////MDSEL9			//
	{0x3847,0x00},
	{0x384A,0x00},////MDSEL10
	{0x384B,0x00},


	{0x3001,0x00},// MDCHGRST    CLPSQRST  //
	{0x3008,0x00},//SMD
	{0x300A,0x00},//PGC				    	//
	{0x300B,0x00},//PGC						//
	{0x3012,0x00},//DGAIN					//
	{0x3017,0xAB},//SYNCDRV
	{0x302B,0x00},//REGHOLD
	{0x3042,0x20},//BLKLEVEL				//
	{0x3061,0x00},//LESS_SHUT Shutter less mode switch
	{0x3092,0x00},//MCOVGAIN				//
	{0x30DD,0x00},//VWIDCUTEN
	{0x30DE,0x00},//VWIDCUT
	{0x30DF,0x00},//VWIDCUT
	{0x30E0,0x00},// VWINPOS
	{0x30E1,0x00},// VWINPOS
	{0x35E5,0x00},//CLKDIVEN   SYSCLKEN
	{0x378C,0x00}//EBDDATAEN


};
CySensorReg16_t SENSOR_ReadoutmodeSettings16 []=
{



};
					
/* Configuration settings for 2nd stabilization period */
CySensorReg8_t SENSOR_2ndstablization8[] ={///After 10ms of 1st stab
		{0x3000,0x02},
		{0x35E5,0x92},
		{0x35E5,0x9A},
		{0x3000,0x00}


};

/* Configuration settings for 3rd stabilization period */
CySensorReg8_t SENSOR_3rdstablization8[]={///After 7ms of 1st stab
		{0x3033,0x20},
		{0x3017,0xA8}


};



////// standby //////
CySensorReg8_t SENSOR_standby8[]={///After 7ms of 1st stab
        {0x3000,0x12}

};


CySensorReg16_t SENSOR_Resolution0ConfigurationSettings [] =
{

};


CyU3PReturnStatus_t I2C_SensorRead(uint16_t regAddr, uint8_t count, uint8_t *buf)
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    CyU3PI2cPreamble_t preamble;
    uint8_t cnt=0;

    for(cnt=0; cnt<3 ; cnt++)
    {
        preamble.buffer[1] = CY_U3P_GET_MSB (regAddr);
        preamble.buffer[2] = CY_U3P_GET_LSB (regAddr);
        preamble.buffer[3] = SENSOR_I2C_READ_ADDRESS; /* Slave address: Read operation */
        preamble.buffer[0] = SENSOR_I2C_WRITE_ADDRESS; /* Slave address: write operation */
        preamble.length = 4;
        preamble.ctrlMask = 0x0004;

        status = CyU3PI2cReceiveBytes (&preamble, buf, count,0);
        CyU3PThreadSleep(1);
        if (status == CY_U3P_SUCCESS)
        {
            break;
        }
#ifdef SENSOR_DEBUG
        else
            CyU3PDebugPrint(4,"\r\nImageSensorSensorRead Failed addr=0x%x",regAddr);
#endif
    }
    return status;
}
CyU3PReturnStatus_t I2C_SensorWrite(uint16_t regAddr, uint16_t count, uint8_t *buf)
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    CyU3PI2cPreamble_t preamble;
    uint8_t cnt=0;

    for(cnt=0; cnt<3 ; cnt++)
    {
        preamble.buffer[1] = CY_U3P_GET_MSB (regAddr);
        preamble.buffer[2] = CY_U3P_GET_LSB (regAddr);
        preamble.buffer[0] = SENSOR_I2C_WRITE_ADDRESS; /* Slave address: write operation */
        preamble.length = 3;
        preamble.ctrlMask = 0x0000;

        status = CyU3PI2cTransmitBytes (&preamble, buf, count,0);
        CyU3PThreadSleep(1);
        if (status == CY_U3P_SUCCESS)
        {
            break;
        }
#ifdef SENSOR_DEBUG
        else
            CyU3PDebugPrint(4,"\r\nImageSensorSensorWrite Failed addr=0x%x",regAddr);
#endif
    }
    return status;
}

/*Use the below function to set/clear fields of 16-bit register*/
void SENSOR_WR_FEILD(uint16_t RegAddr, uint16_t Field, CyBool_t FieldValue)
{
    uint8_t WRBuff[2];
    uint8_t RDBuff[2];
    uint16_t CamRegVal_16;

    I2C_SensorRead(RegAddr, 2, RDBuff);
    CamRegVal_16 = RDBuff[1];
    CamRegVal_16 |=RDBuff[0]<<8;

    if (FieldValue == 0)
    	CamRegVal_16 = (CamRegVal_16 & (~(Field)));
    else if (FieldValue == 1)
    	CamRegVal_16 = (CamRegVal_16 | (Field));

    WRBuff[0] = (CamRegVal_16 & 0xFF00)>>8;
    WRBuff[1] = (CamRegVal_16 & 0x00FF);
#ifdef SENSOR_DEBUG
    CyU3PDebugPrint(4,"\r0x%x = 0x%x 0x%x\r\n",RegAddr,WRBuff[0], WRBuff[1]);
#endif

    I2C_SensorRead(RegAddr, 2, WRBuff);
}

/*Use the below function to write to 8-bit register*/
CyU3PReturnStatus_t SENSOR_WriteConfigurationSettings_Reg8 (CySensorReg8_t * configSettings, uint16_t configSettingsSize)
{
    uint16_t regCounter = 0;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    uint8_t rwBuffer[2];
#ifdef SENSOR_DEBUG
    CyU3PDebugPrint (4,"\r\nWriting Configuration Settings:");
#endif

    for (regCounter = 0; ((regCounter < configSettingsSize) && (status == CY_U3P_SUCCESS)); regCounter++)
    {
#ifdef SENSOR_DEBUG
        CyU3PDebugPrint (4,"\r\nRegister = 0x%x: Value = 0x%x",configSettings[regCounter].regAddr, 
                configSettings[regCounter].regValue);
#endif
        
		rwBuffer[0] = ((configSettings[regCounter].regValue)&0xFF);
        
		status = I2C_SensorWrite (configSettings[regCounter].regAddr, 1, &rwBuffer);
//
    }

    return status;
}

/*Use the below function to write to 16-bit register*/
CyU3PReturnStatus_t SENSOR_WriteConfigurationSettings_Reg16 (CySensorReg16_t * configSettings, uint16_t configSettingsSize)
{
    uint16_t regCounter = 0;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    uint8_t rwBuffer[2];
#ifdef SENSOR_DEBUG
    CyU3PDebugPrint (4,"\r\nWriting Configuration Settings:");
#endif

    for (regCounter = 0; ((regCounter < configSettingsSize) && (status == CY_U3P_SUCCESS)); regCounter++)
    {
#ifdef SENSOR_DEBUG
        CyU3PDebugPrint (4,"\r\nRegister = 0x%x: Value = 0x%x",configSettings[regCounter].regAddr, 
                configSettings[regCounter].regValue);
#endif
        rwBuffer[0] = (((configSettings[regCounter].regValue)>>8)&0xFF);
		rwBuffer[1] = ((configSettings[regCounter].regValue)&0xFF);
        
        status = I2C_SensorWrite (configSettings[regCounter].regAddr, 2, &rwBuffer); 
    }

    return status;
}

/*Use the below function to write to 32-bit register*/
CyU3PReturnStatus_t SENSOR_WriteConfigurationSettings_Reg32 (CySensorReg32_t * configSettings, uint16_t configSettingsSize)
{
    uint16_t regCounter = 0;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    uint8_t rwBuffer[4];
#ifdef SENSOR_DEBUG
    CyU3PDebugPrint (4,"\r\nWriting Configuration Settings:");
#endif

    for (regCounter = 0; ((regCounter < configSettingsSize) && (status == CY_U3P_SUCCESS)); regCounter++)
    {
#ifdef SENSOR_DEBUG
        CyU3PDebugPrint (4,"\r\nRegister = 0x%x: Value = 0x%x",configSettings[regCounter].regAddr, 
                configSettings[regCounter].regValue);
#endif
        rwBuffer[0] = (((configSettings[regCounter].regValue)>>24)&0xFF);
        rwBuffer[1] = (((configSettings[regCounter].regValue)>>16)&0xFF);
        rwBuffer[2] = (((configSettings[regCounter].regValue)>>8)&0xFF);
		rwBuffer[3] = ((configSettings[regCounter].regValue)&0xFF);
        
        status = I2C_SensorWrite (configSettings[regCounter].regAddr, 4, &rwBuffer); 
    }

    return status;
}

					
/* Configure Sensor for Resolution0*/

CyU3PReturnStatus_t CyCx3_ImageSensor_Set_Resolution0()
{
//    uint16_t configSize;
//    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
//
//    configSize = (sizeof(SENSOR_ReadoutmodeSettings8))/(sizeof(CySensorReg16_t));
//    status =  SENSOR_WriteConfigurationSettings_Reg8(SENSOR_ReadoutmodeSettings8, configSize);
//
//    return status;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
       return status;
}



/*Use the below function to initialize Image Sensor*/
CyU3PReturnStatus_t CyCx3_ImageSensor_Init()
{
	uint16_t configSize;
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

#ifdef SENSOR_DEBUG
    CyU3PDebugPrint(4,"\r\n Sensor Initialization");
#endif
//    SEGGER_RTT_Init();


	configSize = (sizeof(SENSOR_timing_power_ConfigurationSettings8))/(sizeof(CySensorReg16_t));
	//ʼ//
    status = SENSOR_WriteConfigurationSettings_Reg8(SENSOR_timing_power_ConfigurationSettings8, configSize);


	//readout changed//
    configSize = (sizeof(SENSOR_ReadoutmodeSettings8))/(sizeof(CySensorReg16_t));
    status = SENSOR_WriteConfigurationSettings_Reg8(SENSOR_ReadoutmodeSettings8, configSize);


    //10ms delay//
    CyU3PThreadSleep (20);

    configSize = (sizeof(SENSOR_2ndstablization8))/(sizeof(CySensorReg16_t));
    status = SENSOR_WriteConfigurationSettings_Reg8(SENSOR_2ndstablization8, configSize);


    //7ms delay//
    CyU3PThreadSleep (10);

    configSize = (sizeof(SENSOR_3rdstablization8))/(sizeof(CySensorReg16_t));
    status = SENSOR_WriteConfigurationSettings_Reg8(SENSOR_3rdstablization8, configSize);
	/*Include any other I2C write required during Sensor Init here*/

    return status;
}

/*Use the below function to put Image Sensor to Sleep*/
CyU3PReturnStatus_t CyCx3_ImageSensor_Sleep()
{
	uint16_t configSize;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    configSize = (sizeof(SENSOR_standby8))/(sizeof(CySensorReg16_t));
    	//sleep//
    status = SENSOR_WriteConfigurationSettings_Reg8(SENSOR_standby8, configSize);
    return status;
}

/*Use the below function to wakeup Image Sensor*/
CyU3PReturnStatus_t CyCx3_ImageSensor_Wakeup()
{
	uint16_t configSize;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    configSize = (sizeof(SENSOR_timing_power_ConfigurationSettings8))/(sizeof(CySensorReg16_t));
    	//ʼ//
        status = SENSOR_WriteConfigurationSettings_Reg8(SENSOR_timing_power_ConfigurationSettings8, configSize);


    	//readout changed//
        configSize = (sizeof(SENSOR_ReadoutmodeSettings8))/(sizeof(CySensorReg16_t));
        status = SENSOR_WriteConfigurationSettings_Reg8(SENSOR_ReadoutmodeSettings8, configSize);


        //10ms delay//
        CyU3PThreadSleep (20);

            configSize = (sizeof(SENSOR_2ndstablization8))/(sizeof(CySensorReg16_t));
            status = SENSOR_WriteConfigurationSettings_Reg8(SENSOR_2ndstablization8, configSize);


            //7ms delay//
            CyU3PThreadSleep (10);

            configSize = (sizeof(SENSOR_3rdstablization8))/(sizeof(CySensorReg16_t));
            status = SENSOR_WriteConfigurationSettings_Reg8(SENSOR_3rdstablization8, configSize);
   return status;
}


CyU3PReturnStatus_t CyCx3_ImageSensor_Trigger_Autofocus()
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
   return status;
}

CyU3PReturnStatus_t CyCx3_ImageSensor_Set_Format(CyU3PSensorStreamFormat_t format)
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
   return status;
}




